/**
 * Copyright (c) 2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;

/**
 * 
 * Utility class like ServiceEndpointKey for referring to a unique operation in a component.
 * 
 * Not used internally in the CDK but for implementors.
 * 
 * @author Victor Noël - Linagora
 *
 */
public class EndpointOperationKey {

    /**
     * Note: the JBI specification considers that multiple interfaces can be mapped to one endpoint (see
     * {@link ServiceEndpoint}, so it is necessary to discrimate the operation not only with the endpoint name but also
     * with the interface name.
     */
    private final QName interfaceName;

    private final String endpointName;

    private final QName operation;

    public EndpointOperationKey(final Exchange exchange) {
        this(exchange.getEndpoint().getEndpointName(), exchange.getInterfaceName(), exchange.getOperation());
    }

    public EndpointOperationKey(final Consumes consumes) {
        this(consumes.getEndpointName(), consumes.getInterfaceName(), consumes.getOperation());
    }

    public EndpointOperationKey(final Provides provides, final QName operation) {
        this(provides.getEndpointName(), provides.getInterfaceName(), operation);
    }

    public EndpointOperationKey(final String endpoint, final QName interfaceName, final QName operation) {
        assert endpoint != null;
        assert interfaceName != null;
        assert operation != null;
        this.endpointName = endpoint;
        this.interfaceName = interfaceName;
        this.operation = operation;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((endpointName == null) ? 0 : endpointName.hashCode());
        result = prime * result + ((interfaceName == null) ? 0 : interfaceName.hashCode());
        result = prime * result + ((operation == null) ? 0 : operation.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        EndpointOperationKey other = (EndpointOperationKey) obj;
        if (endpointName == null) {
            if (other.endpointName != null)
                return false;
        } else if (!endpointName.equals(other.endpointName))
            return false;
        if (interfaceName == null) {
            if (other.interfaceName != null)
                return false;
        } else if (!interfaceName.equals(other.interfaceName))
            return false;
        if (operation == null) {
            if (other.operation != null)
                return false;
        } else if (!operation.equals(other.operation))
            return false;
        return true;
    }

    @Override
    public String toString() {
        return "EndpointOperationKey [interfaceName=" + interfaceName + ", endpointName=" + endpointName
                + ", operation=" + operation + "]";
    }

    public QName getInterfaceName() {
        return interfaceName;
    }

    public String getEndpointName() {
        return endpointName;
    }

    public QName getOperation() {
        return operation;
    }
}
